[llvm] r202198 - Fix resetting the DataLayout in a Module.

Rafael Espindola rafael.espindola at gmail.com
Tue Feb 25 14:23:04 PST 2014


Author: rafael
Date: Tue Feb 25 16:23:04 2014
New Revision: 202198

URL: http://llvm.org/viewvc/llvm-project?rev=202198&view=rev
Log:
Fix resetting the DataLayout in a Module.

No tool does this currently, but as everything else in a module we should be
able to change its DataLayout.

Most of the fix is in DataLayout to make sure it can be reset properly.

The test uses Module::setDataLayout since the fact that we mutate a DataLayout
is an implementation detail. The module could hold a OwningPtr<DataLayout> and
the DataLayout itself could be immutable.

Thanks to Philip Reames for pushing me in the right direction.

Modified:
    llvm/trunk/include/llvm/IR/DataLayout.h
    llvm/trunk/lib/IR/DataLayout.cpp
    llvm/trunk/lib/IR/Module.cpp
    llvm/trunk/unittests/IR/IRBuilderTest.cpp

Modified: llvm/trunk/include/llvm/IR/DataLayout.h
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/include/llvm/IR/DataLayout.h?rev=202198&r1=202197&r2=202198&view=diff
==============================================================================
--- llvm/trunk/lib/IR/DataLayout.cpp (original)
+++ llvm/trunk/lib/IR/DataLayout.cpp Tue Feb 25 16:23:04 2014
@@ -176,7 +176,9 @@ static const LayoutAlignElem DefaultAlig
   { AGGREGATE_ALIGN, 0, 0, 8 }   // struct
 };
 
-void DataLayout::init(StringRef Desc) {
+void DataLayout::reset(StringRef Desc) {
+  clear();
+
   LayoutMap = 0;
   LittleEndian = false;
   StackNaturalAlign = 0;
@@ -344,12 +346,12 @@ void DataLayout::parseSpecifier(StringRe
   }
 }
 
-DataLayout::DataLayout(const Module *M) {
+DataLayout::DataLayout(const Module *M) : LayoutMap(0) {
   const DataLayout *Other = M->getDataLayout();
   if (Other)
     *this = *Other;
   else
-    init("");
+    reset("");
 }
 
 void
@@ -469,8 +471,16 @@ public:
 
 } // end anonymous namespace
 
+void DataLayout::clear() {
+  LegalIntWidths.clear();
+  Alignments.clear();
+  Pointers.clear();
+  delete static_cast<StructLayoutMap *>(LayoutMap);
+  LayoutMap = 0;
+}
+
 DataLayout::~DataLayout() {
-  delete static_cast<StructLayoutMap*>(LayoutMap);
+  clear();
 }
 
 const StructLayout *DataLayout::getStructLayout(StructType *Ty) const {

Modified: llvm/trunk/lib/IR/Module.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/lib/IR/Module.cpp?rev=202198&r1=202197&r2=202198&view=diff
==============================================================================
--- llvm/trunk/lib/IR/Module.cpp (original)
+++ llvm/trunk/lib/IR/Module.cpp Tue Feb 25 16:23:04 2014
@@ -339,17 +339,21 @@ void Module::addModuleFlag(MDNode *Node)
 }
 
 void Module::setDataLayout(StringRef Desc) {
+  DL.reset(Desc);
+
   if (Desc.empty()) {
     DataLayoutStr = "";
   } else {
-    DL.init(Desc);
     DataLayoutStr = DL.getStringRepresentation();
+    // DataLayoutStr is now equivalent to Desc, but since the representation
+    // is not unique, they may not be identical.
   }
 }
 
 void Module::setDataLayout(const DataLayout *Other) {
   if (!Other) {
     DataLayoutStr = "";
+    DL.reset("");
   } else {
     DL = *Other;
     DataLayoutStr = DL.getStringRepresentation();

Modified: llvm/trunk/unittests/IR/IRBuilderTest.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/unittests/IR/IRBuilderTest.cpp?rev=202198&r1=202197&r2=202198&view=diff
==============================================================================
--- llvm/trunk/unittests/IR/IRBuilderTest.cpp (original)
+++ llvm/trunk/unittests/IR/IRBuilderTest.cpp Tue Feb 25 16:23:04 2014
@@ -108,6 +108,14 @@ TEST_F(IRBuilderTest, LandingPadName) {
   EXPECT_EQ(LP->getName(), "LP");
 }
 
+TEST_F(IRBuilderTest, DataLayout) {
+  OwningPtr<Module> M(new Module("test", Ctx));
+  M->setDataLayout("e-n32");
+  EXPECT_TRUE(M->getDataLayout()->isLegalInteger(32));
+  M->setDataLayout("e");
+  EXPECT_FALSE(M->getDataLayout()->isLegalInteger(32));
+}
+
 TEST_F(IRBuilderTest, GetIntTy) {
   IRBuilder<> Builder(BB);
   IntegerType *Ty1 = Builder.getInt1Ty();





More information about the llvm-commits mailing list
--- llvm/trunk/include/llvm/IR/DataLayout.h (original)
+++ llvm/trunk/include/llvm/IR/DataLayout.h Tue Feb 25 16:23:04 2014
@@ -110,7 +110,7 @@ private:
 
   /// Alignments - Where the primitive type alignment data is stored.
   ///
-  /// @sa init().
+  /// @sa reset().
   /// @note Could support multiple size pointer alignments, e.g., 32-bit
   /// pointers vs. 64-bit pointers by extending LayoutAlignment, but for now,
   /// we don't.
@@ -161,30 +161,35 @@ private:
   /// malformed.
   void parseSpecifier(StringRef LayoutDescription);
 
+  // Free all internal data structures.
+  void clear();
+
 public:
-  /// Constructs a DataLayout from a specification string. See init().
-  explicit DataLayout(StringRef LayoutDescription) { init(LayoutDescription); }
+  /// Constructs a DataLayout from a specification string. See reset().
+  explicit DataLayout(StringRef LayoutDescription) : LayoutMap(0) {
+    reset(LayoutDescription);
+  }
 
   /// Initialize target data from properties stored in the module.
   explicit DataLayout(const Module *M);
 
-  DataLayout(const DataLayout &DL) { *this = DL; }
+  DataLayout(const DataLayout &DL) : LayoutMap(0) { *this = DL; }
 
   DataLayout &operator=(const DataLayout &DL) {
+    clear();
     LittleEndian = DL.isLittleEndian();
     StackNaturalAlign = DL.StackNaturalAlign;
     ManglingMode = DL.ManglingMode;
     LegalIntWidths = DL.LegalIntWidths;
     Alignments = DL.Alignments;
     Pointers = DL.Pointers;
-    LayoutMap = 0;
     return *this;
   }
 
   ~DataLayout();  // Not virtual, do not subclass this class
 
   /// Parse a data layout string (with fallback to default values).
-  void init(StringRef LayoutDescription);
+  void reset(StringRef LayoutDescription);
 
   /// Layout endianness...
   bool isLittleEndian() const { return LittleEndian; }

Modified: llvm/trunk/lib/IR/DataLayout.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/lib/IR/DataLayout.cpp?rev=202198&r1=202197&r2=202198&view=diff
==============================================================================